/*
 * Ben NanoNote board late initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2006 Ingenic Semiconductor, <jlwei@ingenic.cn>
 * Copyright (C) Xiangfu Liu <xiangfu.z@gmail.com>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "nanonote.h"
#include "cpm.h"
#include "gpio.h"

/* Later initialisation functions. */

void gpio_init2()
{
	void *gpio_port_base;

	/*
	Initialise LCD pins.
	gpio_as_slcd_8bit: LCD_D0~LCD_D7, SLCD_CLK, SLCD_RS, SLCD_CS, LCD_DE
	*/

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_LCD);
	jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXFUNS, GPIO_SLCD8_MASK);
	jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXSELC, GPIO_SLCD8_MASK);

	/*
	Initialise MSC pins.
	gpio_as_msc: MSC_CMD, MSC_CLK, MSC_D0 ~ MSC_D3
	*/

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_MSC);
        jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXFUNS, GPIO_MSC_MASK);
        jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXSELC, GPIO_MSC_MASK);
        jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXPES, GPIO_MSC_MASK);

	/* Initialise other pins. */

	/* Enable TP4, TP5 as UART0 (gpio_jtag_to_uart0). */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_JTAG);
	jz4740_gpio_ctrl_set(gpio_port_base, GPIO_PXSELS, 1 << GPIO_JTAG_UART_EN);

	/* Enable pull-up on the power switch. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_POWER);
	jz4740_gpio_as_input(gpio_port_base, GPIO_POWER);
	jz4740_gpio_enable_pull(gpio_port_base, GPIO_POWER);

	/* Enable audio output. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_AUDIO);
	jz4740_gpio_as_output(gpio_port_base, GPIO_AUDIO_EN);
	jz4740_gpio_set_pin(gpio_port_base, GPIO_AUDIO_EN);

	/* Enable LCD signalling. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_LCD);
	jz4740_gpio_as_output(gpio_port_base, GPIO_LCD_CS);
	jz4740_gpio_clear_pin(gpio_port_base, GPIO_LCD_CS);

	/* Enable speaker. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_AMP);
	jz4740_gpio_as_output(gpio_port_base, GPIO_AMP_EN);
	jz4740_gpio_clear_pin(gpio_port_base, GPIO_AMP_EN);

	/* Disable pull-up on SD pins. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_SD);
	jz4740_gpio_as_output(gpio_port_base, GPIO_SDPW_EN);
	jz4740_gpio_disable_pull(gpio_port_base, GPIO_SDPW_EN);
	jz4740_gpio_clear_pin(gpio_port_base, GPIO_SDPW_EN);
	jz4740_gpio_as_input(gpio_port_base, GPIO_SD_DETECT);
	jz4740_gpio_disable_pull(gpio_port_base, GPIO_SD_DETECT);

	/* Enable pull-up on USB detect pin. */

	gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_USB);
	jz4740_gpio_as_input(gpio_port_base, GPIO_USB_DETECT);
	jz4740_gpio_enable_pull(gpio_port_base, GPIO_USB_DETECT);

	/* Employ a PWM channel for the buzzer. */

	jz4740_gpio_as_pwm((void *) GPIO_BASE, 4);
}

void rtc_init()
{
	while ( !__rtc_write_ready());
	__rtc_enable_alarm();	/* enable alarm */

	while ( !__rtc_write_ready());
	REG_RTC_RGR   = 0x00007fff; /* type value */

	while ( !__rtc_write_ready());
	REG_RTC_HWFCR = 0x0000ffe0; /* Power on delay 2s */

	while ( !__rtc_write_ready());
	REG_RTC_HRCR  = 0x00000fe0; /* reset delay 125ms */
}

/* Timer routines. */

unsigned long timestamp;
unsigned long lastdec;

/*
 * Timer without interrupts.
 */

void timer_init()
{
	__tcu_disable_pwm_output(TIMER_CHAN);
	__tcu_select_extalclk(TIMER_CHAN);
	__tcu_select_clk_div256(TIMER_CHAN);
	__tcu_set_count(TIMER_CHAN, 0);
	__tcu_set_half_data(TIMER_CHAN, 0);
	__tcu_set_full_data(TIMER_CHAN, TIMER_FDATA);

	__tcu_mask_half_match_irq(TIMER_CHAN);
	__tcu_mask_full_match_irq(TIMER_CHAN);
	__tcu_start_timer_clock(TIMER_CHAN);
	__tcu_start_counter(TIMER_CHAN);

	jz4740_cpm_start_clock((void *) CPM_BASE);

	lastdec = 0;
	timestamp = 0;
}

/* Timer interrupt activation. */

void timer_init_irq()
{
	__tcu_unmask_full_match_irq(TIMER_CHAN);
	__tcu_clear_full_match_flag(TIMER_CHAN);
	__intc_unmask_irq(TIMER_CHAN_IRQ);
}

void timer_clear()
{
	__intc_ack_irq(TIMER_CHAN_IRQ);
	__tcu_clear_full_match_flag(TIMER_CHAN);
}

/* GPIO interrupt activation. */

void gpio_init_irq()
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_POWER);
	jz4740_gpio_as_irq(gpio_port_base, GPIO_POWER, gpio_irq_level_low);
	__intc_unmask_irq(GPIO_IRQ3);
}

int gpio_have_irq(uint8_t gpio)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, gpio / 32);
	return jz4740_gpio_have_irq(gpio_port_base, gpio % 32);
}

/* Board startup detection. */

int is_started()
{
	return jz4740_cpm_have_clock((void *) CPM_BASE);
}
